@@ -1,4 +1,7 @@
+
 #include "net.h"
+#include "compiler.h"
+#include "publish/avahi.h"
 
 #include <inttypes.h>
 #include <stdlib.h>
@@ -7,18 +10,28 @@
 #include <arpa/inet.h>
 #include <net/if.h>
 
+#ifdef ENABLE_TCPWRAP
+#include <tcpd.h>
+int allow_severity;
+int deny_severity;
+#endif
+
 struct tcp_args {
 	char* address;
 	uint16_t port;
 	char* intf;
+#ifdef ENABLE_AVAHI
+	void *avahi;
+#endif
 };
 
 static
-obex_t* _tcp_init (
-	struct tcp_args* args,
+obex_t* tcp_init (
+	struct net_handler *h,
 	obex_event_t eventcb
 )
 {
+	struct tcp_args* args = h->args;
 	obex_t* handle = OBEX_Init(OBEX_TRANS_INET,eventcb,OBEX_FL_KEEPSERVER);
 	
 	if (!handle)
@@ -31,6 +44,7 @@ obex_t* _tcp_init (
 			struct sockaddr_in6 in6;
 		} addr;
 		char* addrstr = args->address;
+		int af = AF_UNSPEC;
 
 		if (!args->address || strcmp(args->address, "*") == 0)
 			addrstr = "::";
@@ -53,7 +67,7 @@ obex_t* _tcp_init (
 			}
 
 		} else if (inet_pton(AF_INET, args->address, &addr.in4.sin_addr) == 1) {
-			addr.raw.sa_family = AF_INET;
+			addr.raw.sa_family = af = AF_INET;
 			addr.in4.sin_port = htons(args->port);
 
 		} else {
@@ -64,26 +78,75 @@ obex_t* _tcp_init (
 			perror("TcpOBEX_ServerRegister");
 			return NULL;
 
-		} else {
-			fprintf(stderr, "Listening on TCP/%s:%d\n",
-				(args->address? args->address: "*"),
-				args->port);
 		}
+		OBEX_SetTransportMTU(handle, OBEX_MAXIMUM_MTU, OBEX_MAXIMUM_MTU);
+		fprintf(stderr, "Listening on tcp/%s:%d\n",
+			(args->address? args->address: "*"),
+			args->port);
+#ifdef ENABLE_AVAHI
+		args->avahi = obex_avahi_setup(af, args->port, args->intf);
+#endif
+		
 	}
 	return handle;
 }
 
+#ifdef ENABLE_AVAHI
 static
-obex_t* tcp_init(
-	void* arg,
-	obex_event_t eventcb
+void tcp_cleanup (
+	struct net_handler *h
 )
 {
-	return _tcp_init((struct tcp_args*)arg, eventcb);
+	struct tcp_args* args = h->args;
+
+	if (args->avahi)
+		obex_avahi_cleanup(args->avahi);
+}
+#define TCP_CLEANUP_FUNC tcp_cleanup
+#else
+#define TCP_CLEANUP_FUNC NULL
+#endif
+
+static
+int tcp_security_check(
+	struct net_handler __unused *h,
+	obex_t *ptr
+)
+{
+#ifdef ENABLE_TCPWRAP
+	int err = 1;
+	int sock = OBEX_GetFD(ptr);
+	struct sockaddr_in6 client, server;
+	socklen_t len;
+	struct request_info req;
+
+	len = sizeof(client);
+	err = getpeername(sock, (struct sockaddr*)&client, &len);
+	if (err < 0)
+		return 0;
+
+	len = sizeof(server);
+	err = getsockname(sock, (struct sockaddr*)&server, &len);
+	if (err < 0)
+		return 0;
+
+	request_init(&req,
+		     RQ_FILE, sock,
+		     RQ_CLIENT_SIN, client,
+		     RQ_SERVER_SIN, server,
+		     RQ_DAEMON, "obexpushd");
+	fromhost(&req);
+
+	return  hosts_access(&req);
+
+#else
+	return 1;
+#endif
 }
 
 static
 int tcp_get_peer(
+	struct net_handler __unused *h,
 	obex_t* handle,
 	char* buffer,
 	size_t bufsiz
@@ -118,27 +181,38 @@ int tcp_get_peer(
 }
 
 static
-struct net_funcs tcp_funcs = {
+struct net_handler_ops tcp_ops = {
 	.init = tcp_init,
-	.get_peer = tcp_get_peer
+	.cleanup = TCP_CLEANUP_FUNC,
+	.get_peer = tcp_get_peer,
+	.security_check = tcp_security_check,
 };
 
-int tcp_setup(
-	struct net_data* data,
+struct net_handler* tcp_setup(
 	const char* address,
 	uint16_t port
 )
 {
-	struct tcp_args* args = malloc(sizeof(*args));
-	data->arg = args;
-	if (!args)
-		return -errno;
+	struct tcp_args* args;
+	struct net_handler *h = net_handler_alloc(&tcp_ops, sizeof(*args));
 
-	if (address)
+	if (!h)
+		return NULL;
+
+	args = h->args;
+
+	if (address) {
 		args->address = strdup(address);
-	else
+		if (!args->address) {
+			int err = errno;
+			net_handler_cleanup(h);
+			errno = err;
+			return NULL;
+		}
+	} else
 		args->address = NULL;
 	args->port = port;
-	data->funcs = &tcp_funcs;
-	return 0;
+	args->intf = NULL;
+
+	return h;
 }
